#!/usr/bin/perl
use strict;
# stripping and rebuilding filenames
use File::Basename;
use File::Copy;
use URI::URL;

## VARIABLES TO CHANGE: ##
my $HOME = "/usr/share/cmcdwnlr2";
my $HEADER = "Comic";
my $BLANK_FILE = "$HOME/blank.bmp";
my $CONF_FILE = "/etc/cmcdwnlr2/comics";
my $DOWN_FILE = "/etc/cmcdwnlr2/download";
## #################### ##


require "$HOME/download.pl";
require "$HOME/import_config.pl";

my %comic_list = read_conf($CONF_FILE);
my $O_DIR = read_file($DOWN_FILE);

# TODO: Add debugging if "-v"
if (@ARGV[0] eq "--list") {
    print "Usage: $0 [COMIC_NAME]\n If you supply a COMIC_NAME, that comic will be run.\n Comic names are:\n";
    foreach (keys(%comic_list)) {
        print "    $_\n";
    }
    print " If none is supplied, all will be run.\n";
} elsif (@ARGV[0]) {
    run_comic(@ARGV[0]);
} else {
    foreach (sort sort_by_tier(keys(%comic_list))) {
        run_comic($_);
    }
}

sub run_comic {
    my $comic = shift;
    mkdir("$O_DIR/$comic");
    if($comic_list{$comic}{"next"}) {
        parse_loop($comic);
    } else {
        # Code for the counting
        my $c=1;
        my @files = glob("$O_DIR/$comic/$HEADER*");
        if($files[0]) {
            $c = $1 if $files[0] =~ /${HEADER}0*([1-9]+[0-9]*).[a-z]+/i;
        }
        my $tier = $comic_list{$comic}{"tier"};
        print "\ncounting $tier:$comic at $c\n";
        my $status = 0;
        my $store = "";
        
        my $url_exp = $comic_list{$comic}{"url"};
        my $last_exp = $comic_list{$comic}{"last"};
        my $skip_few = $comic_list{$comic}{"skip"};
        my $last_count;
        if ($last_exp) { # Then let's get the url right away
            print "Check the first page\n";
            my $html = get_html(convert_url(1, $url_exp));
            $last_count = $1 if ($html =~ /$last_exp/si);
            $last_count = $last_count;
            print "We stop at '$last_count' (now: $c)\n";
        }
        while($status eq 0) {
            if (($last_count) && $c >= ($last_count + 1)) {
                print " Stopping at '$c'\n";
                $status = 2;
            }
            ($status, $store) = comic_number($comic, $c, $last_count) if($status eq 0);
            

            # for websites that occasionally skip pictures...
            if (($status eq 1) && $skip_few) {
                print "There was an error, but we're gonna try a few ($skip_few) ahead\n";
                my $cTemp = $c;
                my $end = $c + $skip_few;
                my @dead;
                until (($status eq 0) || ($cTemp eq $end) ) {
                    $dead[++$#dead] = $cTemp;
                    $cTemp++;
                    ($status, my $store) = comic_number($comic,$cTemp);
                }
                if ($status eq 0) {
                    print "ignore that error. We continue. There's more ahead...\n  reparing:";
                    foreach (@dead) {
                        print "$_...";
                        $store = get_store_path($comic,$_) if ($BLANK_FILE =~ /\.([a-z]*?)$/i); #grabs ext of our blank file
                        copy($BLANK_FILE, "$store.$1") or die "Copying 'blank file' failed: $!";
                    }
                    print "\n";
                }
            }
            $c++;
        }
    }
}

# Returns a 0 or 1
# 0 > it found and downloaded the file at some point in time
# 1 > the file isn't available (or the regex failed)
# 2 > the comic has reached or has exceeded it's end.
sub comic_number {
    my $comic = shift;
    my $c = shift;
    my $last_count = shift;

    my $url_exp = $comic_list{$comic}{"url"};
    my $pic_regex = $comic_list{$comic}{"pic"};
    my $get_text =  $comic_list{$comic}{"get_text"};

    # store is without extension
    # allows more flexibility: ie the file can change type depending on the format stored on the website.
    my $store = get_store_path($comic,$c);
    my @matching = glob("$store*");
    unless ($matching[0]) {
        print "$c:\n";
        my $page_url = convert_url($c,$url_exp);
        print " checking: \"$page_url\"\n";
        my $head = available($page_url);
        if ($head) {
            my ($pic_url, $html);
            if ($pic_regex) {
                $html = get_html($page_url);
                $pic_url = $1 if ( $html =~ /$pic_regex/si );
                unless ($pic_url) {
                    print " no path with regex: \"$pic_regex\"\n";
                    return (1, $store);
                }
            } else {
                $pic_url = $page_url;
            }
            # download_comic saves with the extension of the file based off the type of file
            my $url = url($pic_url, $page_url)->abs->as_string;
            my $saved_path = download_comic($url, $store);
            title_text($html, $pic_url, $saved_path) if $get_text;
            # Check to see if C matches Last_Count
            if (($last_count) && $last_count <= $c) {
                print " We need to stop at '$last_count' and this is '$c'\n";
                return (2, $store);
            } 
        } else {
            print " doesn't exist\n";
            return (1, $store);
        }
    }
    return (0, $store);
}

sub parse_loop {
    my $comic = shift;
    
    my $page_url =  $comic_list{$comic}{"url"};
    my $pic_reg = $comic_list{$comic}{"pic"};
    my $next_reg = $comic_list{$comic}{"next"};
    my $get_text = $comic_list{$comic}{"get_text"};
    my $some_blank = $comic_list{$comic}{"skip"};
        #points out that some pages might be blank, so don't die in the case of a "failed" parse

    my $tier = $comic_list{$comic}{"tier"};
    print "\nparsing $tier:$comic\n";

    # check the folder for a $O_DIR/$comic/next_url file
    my $next_url = "$O_DIR/$comic/next_url";
    if ( -e $next_url ) {
        print "Continuing from last time\n";
        $page_url = read_file($next_url);
    }

    while($page_url) { # Though "true" works just as well...
        print " checking \"$page_url\"\n";


        my $html = get_html($page_url);
        my $pic_url = $1 if ($html =~ /$pic_reg/si);

        my $name = fileparse($pic_url, qr/\.[A-Za-z]*?$/) if $pic_url;
        if ($name && $pic_url) {
            # download_comic doesn't want an extension
            my $store = "$O_DIR/$comic/$name";
            print "  checking \"$store.\$ext\"\n";
            my @matching = glob ("$store*");
            unless (@matching[0]) {
                my $url = url($pic_url, $page_url)->abs->as_string;
                my $saved_path = download_comic($url, $store);
                title_text($html, $pic_url, $saved_path) if $get_text;
            }
            else { print "    exists: '@matching[0]'.\n"; }
        } else {
            print " pic = \"$pic_url\"... next\n";
            return unless $some_blank;
        }

        # parse for next link
        my $new_url = url($1, $page_url)->abs->as_string if ($html =~ /$next_reg/si);
        unless ($new_url) {
            print "no url, so next comic.\n";
            return;
        } if ($new_url eq $page_url) {
            print "urls didn't change, so next comic.\n";
            return;
        } else {
            $page_url = $new_url;
            open(FILE, ">$next_url");
            print(FILE "$page_url");
            print "\n";
        }
    }
}


sub title_text {
    my $html = shift;
    my $image_url = shift;
    my $local_path = shift;

    $html =~ s/></>\n</g;
    my $line = $1 if ($html =~ /(<img.*?$image_url.*?>)/i);
    my $text = $1 if ($line =~ /title="(.*?)"/i);
    $text =~ s/(&#39;|&#8216;|&#8217;)/'/g;    
    $text =~ s/(&quote;|&#8220;|&#8221;)/\\\"/g; # wow. it only gets worse.
    $text =~ s/(&#8230;)/:/g;

    if ($text) {
        print " saving text: \"$text\"\n";
        
        my ($name, $sdir, $ext) = fileparse($local_path, qr/\.[A-Za-z]*/);
        my $save_path = "$sdir"."$name.title"."$ext";
        print " save: \"$save_path\"\n";
        

        my $pic_details = `identify $local_path`;
        my $size = $1 if $pic_details =~ /$local_path [A-Z]+ (\d+?x)\d+? /;

        system("convert -pointsize 12 -size \"$size\" caption:\"$text\" $save_path");
    }
}

sub sort_by_tier {
   $comic_list{$a}{"tier"} <=> $comic_list{$b}{"tier"};
}

sub convert_url {
    my $num = shift;
    my $url = shift;

    if ($url =~ /\{([0-9])\}/) {
        if ( $1 == 0 ) {
            $url =~ s/\{0\}/$num/;
        } else {
            $num = pad($num,$1);
            $url =~ s/\{$1\}/$num/;
        }
    }
    return $url;
}

sub get_store_path {
    my $comic = shift;
    my $num = shift;
    
    my $pad = pad($num,4);
    my $store_path = "$O_DIR/$comic/$HEADER$pad";

    return $store_path;
}

sub pad {
    my $number = shift;
    my $padding = shift;
    $number = sprintf("%${padding}d", $number);
    $number =~ tr/ /0/;
    return $number;
}

